import json
import logging

from zygoat.components import Component
from zygoat.constants import Projects, Phases, Images
from zygoat.utils.shell import docker_run
from zygoat.utils.files import use_dir

from .prettierrc import prettierrc
from .be_pretty import be_pretty
from .pretty_quick import pretty_quick
from .prettierignore import prettierignore

log = logging.getLogger()


class Prettier(Component):
    def create(self):
        log.info("Installing prettier dev dependency into the frontend project")
        docker_run(
            ["yarn", "add", "--dev", "prettier"],
            self.docker_image(Images.NODE),
            Projects.FRONTEND,
        )

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    @property
    def installed(self):
        with use_dir(Projects.FRONTEND):
            with open("package.json") as f:
                return "prettier" in json.load(f).get("devDependencies", {})


prettier = Prettier(sub_components=[prettierrc, be_pretty, pretty_quick, prettierignore])



import json
import logging

from zygoat.components import Component
from zygoat.constants import Projects, Phases, Images
from zygoat.utils.shell import docker_run
from zygoat.utils.files import use_dir

log = logging.getLogger()


class PrettyQuick(Component):
    def create(self):
        log.info("Installing pretty-quick into frontend project")
        docker_run(
            ["yarn", "add", "--dev", "pretty-quick"],
            self.docker_image(Images.NODE),
            Projects.FRONTEND,
        )

        with use_dir(Projects.FRONTEND):
            log.info("Adding pretty-quick lint command")
            with open("package.json") as f:
                data = json.load(f)
                data["scripts"]["pretty-lint"] = "pretty-quick --check --branch"

            log.info("Dumping new frontend package file")
            with open("package.json", "w") as f:
                json.dump(data, f)

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    @property
    def installed(self):
        with use_dir(Projects.FRONTEND):
            with open("package.json") as f:
                data = json.load(f)

                return (
                    "pretty-quick" in data.get("devDependencies", {})
                    and "pretty-lint" in data["scripts"]
                )


pretty_quick = PrettyQuick()



import importlib
import logging

from zygoat.components import Component, FileComponent
from zygoat.config import yaml

from . import resources

log = logging.getLogger()
project_key = "reverse-proxy"
compose_file_name = "docker-compose.yml"


class ReverseProxy(Component):
    pass


class Caddyfile(FileComponent):
    filename = "Caddyfile"
    resource_pkg = resources


class DockerCompose(Component):
    def _dump_config(self, data):
        with open(compose_file_name, "w") as root_config:
            yaml.dump(data, root_config)

    def _load_config(self):
        with open(compose_file_name) as root_config:
            return yaml.load(root_config.read())

    def create(self):
        log.info(f"Reading {compose_file_name} from the repo")

        config = self._load_config()
        config["services"].update(
            yaml.load(importlib.resources.read_text(resources, compose_file_name))
        )

        log.info("Dumping updated docker-compose config")
        self._dump_config(config)

    def delete(self):
        config = self._load_config()

        log.info("Removing proxy service from config")
        del config["services"][project_key]

        log.info("Dumping updated docker-compose config")
        self._dump_config(config)

    @property
    def installed(self):
        services = self._load_config()["services"]

        return project_key in services


reverse_proxy = ReverseProxy(sub_components=[Caddyfile(), DockerCompose()])



import logging

from zygoat.components import Component
from zygoat.constants import Phases, Projects, Images
from zygoat.utils.shell import multi_docker_run

log = logging.getLogger()


class Reformat(Component):
    def create(self):
        multi_docker_run(
            [["pip", "install", "black"], ["black", "."]],
            self.docker_image(Images.PYTHON),
            Projects.BACKEND,
        )

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    @property
    def installed(self):
        return False  # Always run this, no matter what


reformat = Reformat()



from contextlib import contextmanager
import os
import sys

from box import Box
from click import style
from ruamel.yaml import YAML
import logging

from semver import VersionInfo

from .utils.files import find_nearest
from .constants import config_file_name, __version__, ConfigDefaults


yaml = YAML(typ="safe")
yaml.default_flow_style = False
log = logging.getLogger()


class Config(object):
    def __new__(self, initial_data={}):
        """
        Allows us to create a new Config instance that loads itself and returns a box

        :param initial_data: When creating a new config file, add this to the file
        """
        try:
            self.check_version()
            return Config.load()
        except FileNotFoundError:
            log.debug("Config file not found, creating a new one")

        try:
            repo_root = os.path.dirname(find_nearest(".git"))

            with open(os.path.join(repo_root, config_file_name), "w"):
                pass
        except FileNotFoundError:
            log.critical("Zygoat must be run inside a git repository")
            log.critical(
                "Run "
                + style("git init", fg="green", bold=True)
                + " to initialize a git repository"
            )
            sys.exit(1)

        data = {"version": __version__}
        data.update(ConfigDefaults)
        data.update(initial_data)

        Config.dump(Box(data))

        self.check_version()
        return Config.load()

    @classmethod
    @contextmanager
    def settings_file(cls, mode="r"):
        f = open(find_nearest(config_file_name), mode)

        try:
            yield f
        finally:
            f.close()

    @classmethod
    def load(cls):
        with cls.settings_file() as f:
            return Box(yaml.load(f.read()))

    @classmethod
    def dump(cls, data):
        with cls.settings_file(mode="w") as f:
            yaml.dump(data.to_dict(), f)

    @classmethod
    def delete(cls):
        os.remove(find_nearest(config_file_name))

    @classmethod
    def check_version(cls):
        conf = cls.load()
        conf_version = conf.get("version")
        if conf_version is None:
            return

        current = VersionInfo.parse(__version__)
        loaded = VersionInfo.parse(conf_version)

        if current < loaded:
            log.critical(
                "Current version of Zygoat ({}) is older than project version ({}), exiting".format(
                    style(current, bold=True, fg="red"), style(loaded, bold=True, fg="red")
                )
            )
            sys.exit(1)

        log.debug(f"Bumping project version flag from {loaded} to {current}")
        conf.version = __version__
        cls.dump(conf)



from functools import wraps
from importlib import resources as il_resources
import logging
import os

from zygoat.components import Component
from zygoat.constants import Phases
from zygoat.components import resources


log = logging.getLogger()


class FileComponent(Component):
    """
    Use this when you want to create a file component that
    tracks the contents of the file that ``filename`` points to.
    Note that this file must exist in in the supplied resource package.

    Several class properties are available to configure this component:

    :param filename: Name of the file inside of the resource package to copy
    :type filename: str
    :param resource_pkg: The python package that contains the static file to read.
    :param base_path: A path to prepend to the output filename, i.e. ``frontend/static/``
    :type base_path: str, optional
    :param overwrite: If the update phase should recreate the file, defaults to True
    :type overwrite: bool, optional
    """

    resource_pkg = resources
    base_path = "./"
    overwrite = True
    executable = False

    def check_setup(f):
        @wraps(f)
        def wrapper(self, *args, **kwargs):
            if not self.filename:
                raise NotImplementedError("You must specify cls.filename!")
            return f(self, *args, **kwargs)

        return wrapper

    @check_setup
    def create(self):
        log.info(f"Creating {self.path}")
        os.makedirs(self.base_path, exist_ok=True)
        with open(self.path, "w") as f:
            f.write(il_resources.read_text(self.resource_pkg, self.filename))

        if self.executable:
            os.chmod(self.path, 0o755)

    @check_setup
    def update(self):
        self.call_phase(Phases.CREATE, force_create=self.overwrite)

    @check_setup
    def delete(self):
        log.warning(f"Deleting {self.path}")
        os.remove(self.path)
        try:
            os.rmdir(self.base_path)
            log.warning(f"Deleting {self.base_path}")
        except OSError:
            log.warning(f"Skipping {self.base_path}")

    @property
    @check_setup
    def installed(self):
        return os.path.exists(self.path)

    @property
    def path(self):
        return os.path.join(self.base_path, self.filename)



from zygoat.components import FileComponent
from zygoat.constants import Projects

from . import resources


class JestConfig(FileComponent):
    filename = "jest.config.js"
    resource_pkg = resources
    base_path = Projects.FRONTEND
    overwrite = False


jest_config = JestConfig()



from zygoat.components import Component

from .gitignore import gitignore
from .configuration import configuration


class Cypress(Component):
    pass


cypress = Cypress(sub_components=[configuration, gitignore])
